#!/usr/bin/env python3
""" Template for SNLP Assignment 3
    See <https://snlp2019.github.io/a3/> for instructions.

"""

import re, os, glob, argparse
import numpy as np
from keras.models import Sequential
from keras.layers import Dense, Activation, Input
from keras.callbacks import EarlyStopping
from sklearn.metrics import precision_recall_fscore_support, accuracy_score, confusion_matrix

class Encoder():
    """ An input/output encoder class for SNLP assignment 3, exercise 1.
    """
    def __init__(self):
        pass

    def fit(self, conllufile):
        pass

    def transform(self, conllufile):
        pass

    def fit_transform(self, conllufile):
        """ Implementing this in a more "smart" way may avoid some
        double work done in a sequence of fit() and transform().
        """
        self.fit(conllufile)
        return self.transform(conllufile)

    def rev_transform(self, encoded):
        """ Optional: return symbolic dependency labels for a given
        encoded labels (or softmax output).

        You will probably need this in print_scores() below.
        """

def build_model(input_dim, output_dim,
        hidden_layers=(10,10), hidden_activation='relu',
        optimizer='adam'):
    """ SNLP Assignment 3, exercise 2: build a Keras feed-forward model.

        Arguments are
        input_dim           dimension of the input layer
        output_dim          dimension of the output
        hidden_layers       a sequence indicating number of units at each
                            hidden layer
        hidden_activation   activation function for the hidden layers
                            (used for all hidden layers)
        optimizer           the optimizer used for training the network
    """
    pass

def train(model, trn_feat, trn_labels, dev_feat, dev_labels,
        max_epoch=10, verbose=1):
    """ SNLP Assignment 3, exercise 2: build a Keras feed-forward model.
        
        model               a Keras model build using build_model()
        trn_feat            encoded training input (features)
        trn_labels          encoded training output (dependency labels)
        dev_feat            encoded validation features 
        dev_labels          encoded validation output 
        max_epoch           maximum number of epochs to train
        verbose             verbosity level of Keras model's fit() method
    """
    pass


def print_scores(model, encoder, test_file, conf_mat=True,
        two_best=True, treebank_name=""):
    """ SNLP Assignment 3, exercise 3: print performance scores.

        model               a trained Keras model
        endocer             the instance of Encoder() used for training the model
        test_file           the name of the test file
        conf_mat            if True, print the confusion matrix
        two_best            if True, print accuracy for two-best predictions
        treebank_name       if given, prepend to each line displaying the scores
    """
    pass

if __name__ == '__main__':
    argp = argparse.ArgumentParser()
    argp.add_argument('treebanks', nargs="+")
    args = argp.parse_args()

    for tb in args.treebanks:
        trn_file = glob.glob(tb + '/*-train.conllu')[0]
        dev_file = glob.glob(tb + '/*-dev.conllu')[0]
        tst_file = glob.glob(tb + '/*-test.conllu')[0]

        e = Encoder()
        trn_feat, trn_labels = e.fit_transform(trn_file)
        dev_feat, dev_labels = e.transform(dev_file)
        m = build_model(len(e.fset)+1, len(e.dset)+1)
        train(m, trn_feat, trn_labels, dev_feat, dev_labels, verbose=0)

        print_scores(m, e, tst_file, treebank_name=tb, conf_mat=False, two_best=False)
